unit XSysInfo;

{
  TXSystemInformation ver. 1.0 beta
  =================================
  D3 / D4 component to collect system specific information.
  Currently there are 4 information areas - operating system, computer,
  memory and volume (network and display information is expected very soon).

  Freeware.

  Copyright  1999 Roman Stedronsky, Roman.Stedronsky@seznam.cz.

  All rights reserved. You may use this software in an application
  without fee or royalty, provided this copyright notice remains intact.

  properties:

    property OperatingSystem;

      property Platform: string;
      property MajorVersion: string;
      property MinorVersion: string;
      property BuildNumber: string;
      property Extension: string;

    property Computer;

      property ProcessorArchitecture: string;
      property PageSize: string;
      property MinimumApplicationAddress: string;
      property MaximumApplicationAddress: string;
      property ActiveProcessorMask: string;
      property NumberOfProcessors: string;
      property ProcessorType: string;
      property AllocationGranularity: string;
      property ProcessorLevel: string;
      property ProcessorRevision: string;

    property Memory;

      property MemoryLoad: integer;
      property TotalPhysical: integer;
      property AvailablePhysical: integer;
      property TotalPageFile: integer;
      property AvailablePageFile: integer;
      property TotalVirtual: integer;
      property AvailableVirtual: integer;

    property Volume;

      property AvailableDrives: TDrives;

}

interface

uses Classes, Windows, SysUtils;

type

  TCustomInfo = class(TPersistent)
  private
    procedure IntegerWrite(I: integer); virtual;
    procedure StringWrite(S: string); virtual;
  public
    constructor Create(Owner: TComponent); virtual;
    procedure Refresh; virtual;
  end;

  TOperatingSystemInfo = class(TCustomInfo)
  private
    FPlatform: string;
    FMajorVersion: string;
    FMinorVersion: string;
    FBuildNumber: string;
    FExtension: string;
  public
    procedure Refresh; override;
  published
    property Platform: string read FPlatform write StringWrite;
    property MajorVersion: string read FMajorVersion write StringWrite;
    property MinorVersion: string read FMinorVersion write StringWrite;
    property BuildNumber: string read FBuildNumber write StringWrite;
    property Extension: string read FExtension write StringWrite;
//   property DOSVersion: string read FDOSVersion write StringWrite;
  end;

  TComputerInfo = class(TCustomInfo)
  private
    FProcessorArchitecture: string;
    FPageSize: string;
    FMinimumApplicationAddress: string;
    FMaximumApplicationAddress: string;
    FActiveProcessorMask: string;
    FNumberOfProcessors: string;
    FProcessorType: string;
    FAllocationGranularity: string;
    FProcessorLevel: string;
    FProcessorRevision: string;
    FFrequency: string;
  public
    procedure Refresh; override;
  published
    property ProcessorArchitecture: string read FProcessorArchitecture write StringWrite;
    property PageSize: string read FPageSize write StringWrite;
    property MinimumApplicationAddress: string read FMinimumApplicationAddress write StringWrite;
    property MaximumApplicationAddress: string read FMaximumApplicationAddress write StringWrite;
    property ActiveProcessorMask: string read FActiveProcessorMask write StringWrite;
    property NumberOfProcessors: string read FNumberOfProcessors write StringWrite;
    property ProcessorType: string read FProcessorType write StringWrite;
    property AllocationGranularity: string read FAllocationGranularity write StringWrite;
    property ProcessorLevel: string read FProcessorLevel write StringWrite;
    property ProcessorRevision: string read FProcessorRevision write StringWrite;
    property Frequency: string read FFrequency write StringWrite;
  end;

  TMemoryInfo = class(TCustomInfo)
  private
    FMemoryLoad: integer;
    FTotalPhysical: integer;
    FAvailablePhysical: integer;
    FTotalPageFile: integer;
    FAvailablePageFile: integer;
    FTotalVirtual: integer;
    FAvailableVirtual: integer;
  public
    procedure Refresh; override;
  published
    property MemoryLoad: integer read FMemoryLoad write IntegerWrite;
    property TotalPhysical: integer read FTotalPhysical write IntegerWrite;
    property AvailablePhysical: integer read FAvailablePhysical write IntegerWrite;
    property TotalPageFile: integer read FTotalPageFile write IntegerWrite;
    property AvailablePageFile: integer read FAvailablePageFile write IntegerWrite;
    property TotalVirtual: integer read FTotalVirtual write IntegerWrite;
    property AvailableVirtual: integer read FAvailableVirtual write IntegerWrite;
  end;

  TDrivesRange = (A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z);
//  TDrivesRange = (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z);
//  TDrivesRange = 0..25;
  TDrives = set of TDrivesRange;

  TVolumeInfo = class(TCustomInfo)
  private
    FAvailableDrives: TDrives;
    procedure FWriteDrives(D: TDrives);
  public
    procedure Refresh; override;
  published
    property AvailableDrives: TDrives read FAvailableDrives write FWriteDrives;
  end;

  TNetworkInfo = class(TCustomInfo)
  private
  public
  published
  end;

  TDisplayInfo = class(TCustomInfo)
  private
  public
  published
  end;

  TXSystemInformation = class(TComponent)
  private
    FComputerInfo: TComputerInfo;
    FVolumeInfo: TVolumeInfo;
    FOperatingSystemInfo: TOperatingSystemInfo;
    FMemoryInfo: TMemoryInfo;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
  published
    property OperatingSystem: TOperatingSystemInfo read FOperatingSystemInfo write FOperatingSystemInfo;
    property Computer: TComputerInfo read FComputerInfo write FComputerInfo;
    property Volume: TVolumeInfo read FVolumeInfo write FVolumeInfo;
    property Memory: TMemoryInfo read FMemoryInfo write FMemoryInfo;
  end;

procedure Register;

implementation

type
  TProcessorArchType = (
    PROCESSOR_ARCHITECTURE_INTEL,
    PROCESSOR_ARCHITECTURE_MIPS,
    PROCESSOR_ARCHITECTURE_ALPHA,
    PROCESSOR_ARCHITECTURE_PPC);

const
  TPlatformID: array[VER_PLATFORM_WIN32s..VER_PLATFORM_WIN32_NT] of string = ('Windows', 'Windows 95', 'Windows NT');
  TProcessorArchitecture: array[0..3] of string = ('Intel', 'MIPS', 'Aplha', 'PPC');

resourcestring
  rsRevisionFormat = 'Model %u Stepping %u';

constructor TCustomInfo.Create(Owner: TComponent);
begin
  inherited Create;
  Refresh;
end;

procedure TCustomInfo.StringWrite(S: string);
begin
end;

procedure TCustomInfo.IntegerWrite(I: integer);
begin
end;

procedure TCustomInfo.Refresh;
begin
end;

procedure TOperatingSystemInfo.Refresh;
var
  OSVersion: TOSVersionInfo;
begin
  OSVersion.dwOSVersionInfoSize := SizeOf(OSVersion);
  GetVersionEx(OSVersion);
  FPlatform := TPlatformID[OSVersion.dwPlatformID];
  FMajorVersion := IntToStr(OSVersion.dwMajorVersion);
  FMinorVersion := IntToStr(OSVersion.dwMinorVersion);
  FBuildNumber := IntToStr(OSVersion.dwBuildNumber and $FFFF);
  if SystemParametersInfo(SPI_GetWindowsExtension, 1, nil, 0) then
    FExtension := 'PLUS!'
  else
    FExtension := OSVersion.szCSDVersion;
end;

procedure TComputerInfo.Refresh;
var
  SysInfo: TSystemInfo;
  function GetCpuSpeed: Extended;
  var
    Time: DWORD;
    StartHi, StartLo, EndHi, EndLo: DWORD;
  begin
    Time := GetTickCount;
    asm
    DB 0FH
    DB 031H
    mov StartHi,edx
    mov StartLo,eax
    end;
    while GetTickCount < (Time + 1000) do begin end;
    asm
    DB 0FH
    DB 031H
    mov EndHi,edx
    mov EndLo,eax
    end;
    Result := (EndLo - StartLo + 4294967296 * (EndHi - StartHi)) / 1000000;
  end;
begin
  ZeroMemory(@SysInfo, SizeOf(SysInfo));
  GetSystemInfo(SysInfo);
  FProcessorArchitecture := TProcessorArchitecture[SysInfo.wProcessorArchitecture];
  FPageSize := IntToStr(SysInfo.dwPageSize);
  FMinimumApplicationAddress := IntToStr(Integer(SysInfo.lpMinimumApplicationAddress));
  FMaximumApplicationAddress := IntToStr(Integer(SysInfo.lpMaximumApplicationAddress));
  FActiveProcessorMask := IntToStr(SysInfo.dwActiveProcessorMask);
  FNumberOfProcessors := IntToStr(SysInfo.dwNumberOfProcessors);
  FProcessorType := IntToStr(SysInfo.dwProcessorType);
  FAllocationGranularity := IntToStr(SysInfo.dwAllocationGranularity);
  FProcessorLevel := IntToStr(SysInfo.wProcessorLevel);
  case SysInfo.dwProcessorType of
    386, 486: FProcessorRevision := IntToStr(SysInfo.wProcessorRevision);
    586: FProcessorRevision := Format(rsRevisionFormat,
        [SysInfo.wProcessorRevision shr 8, SysInfo.wProcessorRevision - ((SysInfo.wProcessorRevision shr 8) shl 8)]);
  end;
  FFrequency := Format('%.3f', [GetCPUSpeed]);
end;

procedure TMemoryInfo.Refresh;
var
  MemoryStatus: TMemoryStatus;
begin
  ZeroMemory(@MemoryStatus, SizeOf(MemoryStatus));
  MemoryStatus.dwLength := SizeOf(MemoryStatus);
  GlobalMemoryStatus(MemoryStatus);
  FMemoryLoad := MemoryStatus.dwMemoryLoad;
  FTotalPhysical := MemoryStatus.dwTotalPhys;
  FAvailablePhysical := MemoryStatus.dwAvailPhys;
  FTotalPageFile := MemoryStatus.dwTotalPageFile;
  FAvailablePageFile := MemoryStatus.dwAvailPageFile;
  FTotalVirtual := MemoryStatus.dwTotalVirtual;
  FAvailableVirtual := MemoryStatus.dwAvailVirtual;
end;

procedure TVolumeInfo.Refresh;
var
  RootPathName: array[0..255] of char;
  VolumeName: array[0..20] of char;
  VolumeSerialNumber: PDWORD;
  MaximumComponentLength: DWORD;
  FileSystemFlags: DWORD;
  FileSystemName: array[0..20] of char;
  SectorsPerCluster: DWORD;
  BytesPerSector: DWORD;
  FreeClusters: DWORD;
  NumberOfClusters: DWORD;
  BytesTotal: integer;
  BytesFree: integer;
  pom: string;
begin
  Integer(FAvailableDrives) := GetLogicalDrives;
  GetDir(0, pom);
  StrPLCopy(RootPathName, pom, 3);
  New(VolumeSerialNumber);
  GetVolumeInformation(RootPathName, VolumeName, 20, VolumeSerialNumber, MaximumComponentLength,
    FileSystemFlags, FileSystemName, 20);
//  FileSysFlags := GetVolumeFlags(Drive);
{
  FILE_CASE_SENSITIVE_SEARCH          = $00000001;
  FILE_CASE_PRESERVED_NAMES           = $00000002;
  FILE_UNICODE_ON_DISK                = $00000004;
  FILE_PERSISTENT_ACLS                = $00000008;
  FILE_FILE_COMPRESSION               = $00000010;
  FILE_VOLUME_IS_COMPRESSED           = $00008000;

  FS_CASE_IS_PRESERVED = FILE_CASE_PRESERVED_NAMES;
  FS_CASE_SENSITIVE = FILE_CASE_SENSITIVE_SEARCH;
  FS_UNICODE_STORED_ON_DISK = FILE_UNICODE_ON_DISK;
  FS_PERSISTENT_ACLS = FILE_PERSISTENT_ACLS;
  FS_VOL_IS_COMPRESSED = FILE_VOLUME_IS_COMPRESSED;
  FS_FILE_COMPRESSION = FILE_FILE_COMPRESSION;

  if FS_CASE_IS_PRESERVED and FileSysFlags <> 0 Then
  Label1.Caption := 'FS_CASE_IS_PRESERVED';
 If FS_CASE_SENSITIVE AND FileSysFlags <> 0 Then
  Label2.Caption := 'FS_CASE_SENSITIVE';
 If FS_UNICODE_STORED_ON_DISK AND FileSysFlags <> 0 Then
  Label3.Caption := 'FS_UNICODE_STORED_ON_DISK';
 If FS_PERSISTENT_ACLS AND FileSysFlags <> 0 Then
  Label4.Caption := 'FS_PERSISTENT_ACLS'

 DriveLetter := Char(Drive + $41);
 DriveType   := GetDriveType(PChar(DriveLetter + ':\'));
 Case DriveType of
  0               : Result := '?';
  1               : Result := 'Path does not exists';
  DRIVE_REMOVABLE : Result := 'Removable';
  DRIVE_FIXED     : Result := 'Fixed';
  DRIVE_REMOTE    : Result := 'Remote';
  DRIVE_CDROM     : Result := 'CD-ROM';
  DRIVE_RAMDISK   : Result := 'RAMDISK'

function DriveExists1(Drive : Byte) : Boolean;
var
 LogDrives : set of 0..25;
begin
 Integer (LogDrives) := GetLogicalDrives;
 Result := Drive IN LogDrives;
 // alternative  Result := Boolean(GetLogicalDrives AND(1 SHL Drive))
end;
}
  GetDiskFreeSpace(RootPathName, SectorsPerCluster, BytesPerSector, FreeClusters, NumberOfClusters);
  BytesTotal := NumberOfClusters * SectorsPerCluster * BytesPerSector;
  BytesFree := FreeClusters * SectorsPerCluster * BytesPerSector;
end;

procedure TVolumeInfo.FWriteDrives(D: TDrives);
begin
end;

constructor TXSystemInformation.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOperatingSystemInfo := TOperatingSystemInfo.Create(self);
  FComputerInfo := TComputerInfo.Create(self);
  FVolumeInfo := TVolumeInfo.Create(self);
  FMemoryInfo := TMemoryInfo.Create(self);
end;

destructor TXSystemInformation.Destroy;
begin
  FOperatingSystemInfo.Free;
  FComputerInfo.Free;
  FVolumeInfo.Free;
  FMemoryInfo.Free;
  inherited Destroy;
end;

procedure TXSystemInformation.Refresh;
begin
  FOperatingSystemInfo.Refresh;
  FComputerInfo.Refresh;
  FVolumeInfo.Refresh;
  FMemoryInfo.Refresh;
end;

procedure Register;
begin
  RegisterComponents('Extra', [TXSystemInformation]);
end;

end.

